"""Django REST Framework API views for device onboarding.

(c) 2020 Network To Code
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
  http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# from drf_yasg.openapi import Parameter, TYPE_STRING
# from drf_yasg.utils import swagger_auto_schema

from rest_framework import mixins, viewsets

# from rest_framework.decorators import action
# from rest_framework.response import Response

# from utilities.api import IsAuthenticatedOrLoginNotRequired

# from dcim.models import Device, Site, Platform, DeviceRole

from netbox_onboarding.models import OnboardingTask
from netbox_onboarding.filters import OnboardingTaskFilter

# from netbox_onboarding.choices import OnboardingStatusChoices
from .serializers import OnboardingTaskSerializer


class OnboardingTaskView(
    mixins.CreateModelMixin,
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    mixins.DestroyModelMixin,
    viewsets.GenericViewSet,
):
    """Create, check status of, and delete onboarding tasks.

    In-place updates (PUT, PATCH) of tasks are not permitted.
    """

    queryset = OnboardingTask.objects.all()
    filterset_class = OnboardingTaskFilter
    serializer_class = OnboardingTaskSerializer



"""Constants for netbox_onboarding plugin."""

NETMIKO_TO_NAPALM_STATIC = {
    "cisco_ios": "ios",
    "cisco_nxos": "nxos_ssh",
    "arista_eos": "eos",
    "juniper_junos": "junos",
    "cisco_xr": "iosxr",
}



"""Unit tests for netbox_onboarding OnboardingDevice model.

(c) 2020 Network To Code
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
  http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from django.test import TestCase

from dcim.models import Site, DeviceRole, DeviceType, Manufacturer, Device, Interface
from ipam.models import IPAddress

from netbox_onboarding.models import OnboardingTask
from netbox_onboarding.models import OnboardingDevice
from netbox_onboarding.choices import OnboardingStatusChoices


class OnboardingDeviceModelTestCase(TestCase):
    """Test the Onboarding models."""

    def setUp(self):
        """Setup objects for Onboarding Model tests."""
        self.site = Site.objects.create(name="USWEST", slug="uswest")
        manufacturer = Manufacturer.objects.create(name="Juniper", slug="juniper")
        device_role = DeviceRole.objects.create(name="Firewall", slug="firewall")
        device_type = DeviceType.objects.create(slug="srx3600", model="SRX3600", manufacturer=manufacturer)

        self.device = Device.objects.create(
            device_type=device_type, name="device1", device_role=device_role, site=self.site,
        )

        intf = Interface.objects.create(name="test_intf", device=self.device)

        primary_ip = IPAddress.objects.create(address="10.10.10.10/32")
        intf.ip_addresses.add(primary_ip)

        self.device.primary_ip4 = primary_ip
        self.device.save()

        self.succeeded_task1 = OnboardingTask.objects.create(
            ip_address="10.10.10.10",
            site=self.site,
            status=OnboardingStatusChoices.STATUS_SUCCEEDED,
            created_device=self.device,
        )

        self.succeeded_task2 = OnboardingTask.objects.create(
            ip_address="10.10.10.10",
            site=self.site,
            status=OnboardingStatusChoices.STATUS_SUCCEEDED,
            created_device=self.device,
        )

        self.failed_task1 = OnboardingTask.objects.create(
            ip_address="10.10.10.10",
            site=self.site,
            status=OnboardingStatusChoices.STATUS_FAILED,
            created_device=self.device,
        )

        self.failed_task2 = OnboardingTask.objects.create(
            ip_address="10.10.10.10",
            site=self.site,
            status=OnboardingStatusChoices.STATUS_FAILED,
            created_device=self.device,
        )

    def test_onboardingdevice_autocreated(self):
        """Verify that OnboardingDevice is auto-created."""
        onboarding_device = OnboardingDevice.objects.get(device=self.device)
        self.assertEqual(self.device, onboarding_device.device)

    def test_last_check_attempt_date(self):
        """Verify OnboardingDevice last attempt."""
        onboarding_device = OnboardingDevice.objects.get(device=self.device)
        self.assertEqual(onboarding_device.last_check_attempt_date, self.failed_task2.created)

    def test_last_check_successful_date(self):
        """Verify OnboardingDevice last success."""
        onboarding_device = OnboardingDevice.objects.get(device=self.device)
        self.assertEqual(onboarding_device.last_check_successful_date, self.succeeded_task2.created)

    def test_status(self):
        """Verify OnboardingDevice status."""
        onboarding_device = OnboardingDevice.objects.get(device=self.device)
        self.assertEqual(onboarding_device.status, self.failed_task2.status)

    def test_last_ot(self):
        """Verify OnboardingDevice last ot."""
        onboarding_device = OnboardingDevice.objects.get(device=self.device)
        self.assertEqual(onboarding_device.last_ot, self.failed_task2)



"""Django views for device onboarding.

(c) 2020 Network To Code
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
  http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import logging

from django.shortcuts import get_object_or_404, render
from django.views.generic import View

from utilities.views import BulkDeleteView, BulkImportView, ObjectEditView, ObjectListView

from .release import NETBOX_RELEASE_CURRENT, NETBOX_RELEASE_29

from .filters import OnboardingTaskFilter
from .forms import OnboardingTaskForm, OnboardingTaskFilterForm, OnboardingTaskFeedCSVForm
from .models import OnboardingTask
from .tables import OnboardingTaskTable, OnboardingTaskFeedBulkTable

logger = logging.getLogger("rq.worker")


if NETBOX_RELEASE_CURRENT < NETBOX_RELEASE_29:
    from django.contrib.auth.mixins import PermissionRequiredMixin  # pylint: disable=ungrouped-imports

    class ReleaseMixinOnboardingTaskView(PermissionRequiredMixin, View):
        """Release Mixin View for presenting a single OnboardingTask."""

        permission_required = "netbox_onboarding.view_onboardingtask"

    class ReleaseMixinOnboardingTaskListView(PermissionRequiredMixin):
        """Release Mixin View for listing all extant OnboardingTasks."""

        permission_required = "netbox_onboarding.view_onboardingtask"

    class ReleaseMixinOnboardingTaskCreateView(PermissionRequiredMixin):
        """Release Mixin View for creating a new OnboardingTask."""

        permission_required = "netbox_onboarding.add_onboardingtask"

    class ReleaseMixinOnboardingTaskBulkDeleteView(PermissionRequiredMixin):
        """Release Mixin View for deleting one or more OnboardingTasks."""

        permission_required = "netbox_onboarding.delete_onboardingtask"

    class ReleaseMixinOnboardingTaskFeedBulkImportView(PermissionRequiredMixin):
        """Release Mixin View for bulk-importing a CSV file to create OnboardingTasks."""

        permission_required = "netbox_onboarding.add_onboardingtask"


else:
    from utilities.views import ObjectView  # pylint: disable=ungrouped-imports, no-name-in-module

    class ReleaseMixinOnboardingTaskView(ObjectView):
        """Release Mixin View for presenting a single OnboardingTask."""

    class ReleaseMixinOnboardingTaskListView:
        """Release Mixin View for listing all extant OnboardingTasks."""

    class ReleaseMixinOnboardingTaskCreateView:
        """Release Mixin View for creating a new OnboardingTask."""

    class ReleaseMixinOnboardingTaskBulkDeleteView:
        """Release Mixin View for deleting one or more OnboardingTasks."""

    class ReleaseMixinOnboardingTaskFeedBulkImportView:
        """Release Mixin View for bulk-importing a CSV file to create OnboardingTasks."""


class OnboardingTaskView(ReleaseMixinOnboardingTaskView):
    """View for presenting a single OnboardingTask."""

    queryset = OnboardingTask.objects.all()

    def get(self, request, pk):  # pylint: disable=invalid-name, missing-function-docstring
        """Get request."""
        onboardingtask = get_object_or_404(self.queryset, pk=pk)

        return render(request, "netbox_onboarding/onboardingtask.html", {"onboardingtask": onboardingtask,})


class OnboardingTaskListView(ReleaseMixinOnboardingTaskListView, ObjectListView):
    """View for listing all extant OnboardingTasks."""

    queryset = OnboardingTask.objects.all().order_by("-id")
    filterset = OnboardingTaskFilter
    filterset_form = OnboardingTaskFilterForm
    table = OnboardingTaskTable
    template_name = "netbox_onboarding/onboarding_tasks_list.html"


class OnboardingTaskCreateView(ReleaseMixinOnboardingTaskCreateView, ObjectEditView):
    """View for creating a new OnboardingTask."""

    model = OnboardingTask
    queryset = OnboardingTask.objects.all()
    model_form = OnboardingTaskForm
    template_name = "netbox_onboarding/onboarding_task_edit.html"
    default_return_url = "plugins:netbox_onboarding:onboardingtask_list"


class OnboardingTaskBulkDeleteView(ReleaseMixinOnboardingTaskBulkDeleteView, BulkDeleteView):
    """View for deleting one or more OnboardingTasks."""

    queryset = OnboardingTask.objects.filter()  # TODO: can we exclude currently-running tasks?
    table = OnboardingTaskTable
    default_return_url = "plugins:netbox_onboarding:onboardingtask_list"


class OnboardingTaskFeedBulkImportView(ReleaseMixinOnboardingTaskFeedBulkImportView, BulkImportView):
    """View for bulk-importing a CSV file to create OnboardingTasks."""

    queryset = OnboardingTask.objects.all()
    model_form = OnboardingTaskFeedCSVForm
    table = OnboardingTaskFeedBulkTable
    default_return_url = "plugins:netbox_onboarding:onboardingtask_list"



"""Plugin declaration for netbox_onboarding.

(c) 2020 Network To Code
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
  http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

__version__ = "2.0.0"

from extras.plugins import PluginConfig


class OnboardingConfig(PluginConfig):
    """Plugin configuration for the netbox_onboarding plugin."""

    name = "netbox_onboarding"
    verbose_name = "Device Onboarding"
    version = __version__
    author = "Network to Code"
    description = "A plugin for NetBox to easily onboard new devices."
    base_url = "onboarding"
    required_settings = []
    min_version = "2.8.1"
    default_settings = {
        "create_platform_if_missing": True,
        "create_manufacturer_if_missing": True,
        "create_device_type_if_missing": True,
        "create_device_role_if_missing": True,
        "default_device_role": "network",
        "default_device_role_color": "FF0000",
        "default_management_interface": "PLACEHOLDER",
        "default_management_prefix_length": 0,
        "default_device_status": "active",
        "create_management_interface_if_missing": True,
        "skip_device_type_on_update": False,
        "skip_manufacturer_on_update": False,
        "platform_map": {},
        "onboarding_extensions_map": {"ios": "netbox_onboarding.onboarding_extensions.ios",},
        "object_match_strategy": "loose",
    }
    caching_config = {}


config = OnboardingConfig  # pylint:disable=invalid-name



"""Unit tests for netbox_onboarding.onboard module and its classes.

(c) 2020 Network To Code
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
  http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from django.conf import settings
from django.test import TestCase
from django.utils.text import slugify
from dcim.models import Site, Manufacturer, DeviceType, DeviceRole, Device, Interface, Platform
from ipam.models import IPAddress

# from netbox_onboarding.netbox_keeper import NetdevKeeper
from netbox_onboarding.exceptions import OnboardException
from netbox_onboarding.netbox_keeper import NetboxKeeper

PLUGIN_SETTINGS = settings.PLUGINS_CONFIG["netbox_onboarding"]


class NetboxKeeperTestCase(TestCase):
    """Test the NetboxKeeper Class."""

    def setUp(self):
        """Create a superuser and token for API calls."""
        self.site1 = Site.objects.create(name="USWEST", slug="uswest")

    def test_ensure_device_manufacturer_strict_missing(self):
        """Verify ensure_device_manufacturer function when Manufacturer object is not present."""
        PLUGIN_SETTINGS["object_match_strategy"] = "strict"
        onboarding_kwargs = {
            "netdev_hostname": "device1",
            "netdev_nb_role_slug": PLUGIN_SETTINGS["default_device_role"],
            "netdev_vendor": "Cisco",
            "netdev_model": "CSR1000v",
            "netdev_nb_site_slug": self.site1.slug,
        }

        nbk = NetboxKeeper(**onboarding_kwargs)

        with self.assertRaises(OnboardException) as exc_info:
            nbk.ensure_device_manufacturer(create_manufacturer=False)
            self.assertEqual(exc_info.exception.message, "ERROR manufacturer not found: Cisco")
            self.assertEqual(exc_info.exception.reason, "fail-config")

        nbk.ensure_device_manufacturer(create_manufacturer=True)
        self.assertIsInstance(nbk.nb_manufacturer, Manufacturer)
        self.assertEqual(nbk.nb_manufacturer.slug, slugify(onboarding_kwargs["netdev_vendor"]))

    def test_ensure_device_manufacturer_loose_missing(self):
        """Verify ensure_device_manufacturer function when Manufacturer object is not present."""
        PLUGIN_SETTINGS["object_match_strategy"] = "loose"
        onboarding_kwargs = {
            "netdev_hostname": "device1",
            "netdev_nb_role_slug": PLUGIN_SETTINGS["default_device_role"],
            "netdev_vendor": "Cisco",
            "netdev_model": "CSR1000v",
            "netdev_nb_site_slug": self.site1.slug,
        }

        nbk = NetboxKeeper(**onboarding_kwargs)

        with self.assertRaises(OnboardException) as exc_info:
            nbk.ensure_device_manufacturer(create_manufacturer=False)
            self.assertEqual(exc_info.exception.message, "ERROR manufacturer not found: Cisco")
            self.assertEqual(exc_info.exception.reason, "fail-config")

        nbk.ensure_device_manufacturer(create_manufacturer=True)
        self.assertIsInstance(nbk.nb_manufacturer, Manufacturer)
        self.assertEqual(nbk.nb_manufacturer.slug, slugify(onboarding_kwargs["netdev_vendor"]))

    def test_ensure_device_type_strict_missing(self):
        """Verify ensure_device_type function when DeviceType object is not present."""
        PLUGIN_SETTINGS["object_match_strategy"] = "strict"
        onboarding_kwargs = {
            "netdev_hostname": "device1",
            "netdev_nb_role_slug": PLUGIN_SETTINGS["default_device_role"],
            "netdev_vendor": "Cisco",
            "netdev_model": "CSR1000v",
            "netdev_nb_site_slug": self.site1.slug,
        }

        nbk = NetboxKeeper(**onboarding_kwargs)
        nbk.nb_manufacturer = Manufacturer.objects.create(name="Cisco", slug="cisco")

        with self.assertRaises(OnboardException) as exc_info:
            nbk.ensure_device_type(create_device_type=False)
            self.assertEqual(exc_info.exception.message, "ERROR device type not found: CSR1000v")
            self.assertEqual(exc_info.exception.reason, "fail-config")

        nbk.ensure_device_type(create_device_type=True)
        self.assertIsInstance(nbk.nb_device_type, DeviceType)
        self.assertEqual(nbk.nb_device_type.slug, slugify(onboarding_kwargs["netdev_model"]))

    def test_ensure_device_type_loose_missing(self):
        """Verify ensure_device_type function when DeviceType object is not present."""
        PLUGIN_SETTINGS["object_match_strategy"] = "loose"
        onboarding_kwargs = {
            "netdev_hostname": "device1",
            "netdev_nb_role_slug": PLUGIN_SETTINGS["default_device_role"],
            "netdev_vendor": "Cisco",
            "netdev_model": "CSR1000v",
            "netdev_nb_site_slug": self.site1.slug,
        }

        nbk = NetboxKeeper(**onboarding_kwargs)
        nbk.nb_manufacturer = Manufacturer.objects.create(name="Cisco", slug="cisco")

        with self.assertRaises(OnboardException) as exc_info:
            nbk.ensure_device_type(create_device_type=False)
            self.assertEqual(exc_info.exception.message, "ERROR device type not found: CSR1000v")
            self.assertEqual(exc_info.exception.reason, "fail-config")

        nbk.ensure_device_type(create_device_type=True)
        self.assertIsInstance(nbk.nb_device_type, DeviceType)
        self.assertEqual(nbk.nb_device_type.slug, slugify(onboarding_kwargs["netdev_model"]))

    def test_ensure_device_type_strict_present(self):
        """Verify ensure_device_type function when DeviceType object is already present."""
        PLUGIN_SETTINGS["object_match_strategy"] = "strict"
        manufacturer = Manufacturer.objects.create(name="Juniper", slug="juniper")

        device_type = DeviceType.objects.create(slug="srx3600", model="SRX3600", manufacturer=manufacturer)

        onboarding_kwargs = {
            "netdev_hostname": "device2",
            "netdev_nb_role_slug": PLUGIN_SETTINGS["default_device_role"],
            "netdev_vendor": "Juniper",
            "netdev_nb_device_type_slug": device_type.slug,
            "netdev_nb_site_slug": self.site1.slug,
        }

        nbk = NetboxKeeper(**onboarding_kwargs)
        nbk.nb_manufacturer = manufacturer

        nbk.ensure_device_type(create_device_type=False)
        self.assertEqual(nbk.nb_device_type, device_type)

    def test_ensure_device_type_loose_present(self):
        """Verify ensure_device_type function when DeviceType object is already present."""
        PLUGIN_SETTINGS["object_match_strategy"] = "loose"
        manufacturer = Manufacturer.objects.create(name="Juniper", slug="juniper")

        device_type = DeviceType.objects.create(slug="srx3600", model="SRX3600", manufacturer=manufacturer)

        onboarding_kwargs = {
            "netdev_hostname": "device2",
            "netdev_nb_role_slug": PLUGIN_SETTINGS["default_device_role"],
            "netdev_vendor": "Juniper",
            "netdev_nb_device_type_slug": device_type.slug,
            "netdev_nb_site_slug": self.site1.slug,
        }

        nbk = NetboxKeeper(**onboarding_kwargs)
        nbk.nb_manufacturer = manufacturer

        nbk.ensure_device_type(create_device_type=False)
        self.assertEqual(nbk.nb_device_type, device_type)

    def test_ensure_device_role_not_exist(self):
        """Verify ensure_device_role function when DeviceRole does not already exist."""
        test_role_name = "mytestrole"

        onboarding_kwargs = {
            "netdev_hostname": "device1",
            "netdev_nb_role_slug": test_role_name,
            "netdev_nb_role_color": PLUGIN_SETTINGS["default_device_role_color"],
            "netdev_vendor": "Cisco",
            "netdev_nb_site_slug": self.site1.slug,
        }

        nbk = NetboxKeeper(**onboarding_kwargs)

        with self.assertRaises(OnboardException) as exc_info:
            nbk.ensure_device_role(create_device_role=False)
            self.assertEqual(exc_info.exception.message, f"ERROR device role not found: {test_role_name}")
            self.assertEqual(exc_info.exception.reason, "fail-config")

        nbk.ensure_device_role(create_device_role=True)
        self.assertIsInstance(nbk.nb_device_role, DeviceRole)
        self.assertEqual(nbk.nb_device_role.slug, slugify(test_role_name))

    def test_ensure_device_role_exist(self):
        """Verify ensure_device_role function when DeviceRole exist but is not assigned to the OT."""
        device_role = DeviceRole.objects.create(name="Firewall", slug="firewall")

        onboarding_kwargs = {
            "netdev_hostname": "device1",
            "netdev_nb_role_slug": device_role.slug,
            "netdev_nb_role_color": PLUGIN_SETTINGS["default_device_role_color"],
            "netdev_vendor": "Cisco",
            "netdev_nb_site_slug": self.site1.slug,
        }

        nbk = NetboxKeeper(**onboarding_kwargs)
        nbk.ensure_device_role(create_device_role=False)

        self.assertEqual(nbk.nb_device_role, device_role)

    #
    def test_ensure_device_role_assigned(self):
        """Verify ensure_device_role function when DeviceRole exist and is already assigned."""
        device_role = DeviceRole.objects.create(name="Firewall", slug="firewall")

        onboarding_kwargs = {
            "netdev_hostname": "device1",
            "netdev_nb_role_slug": device_role.slug,
            "netdev_nb_role_color": PLUGIN_SETTINGS["default_device_role_color"],
            "netdev_vendor": "Cisco",
            "netdev_nb_site_slug": self.site1.slug,
        }

        nbk = NetboxKeeper(**onboarding_kwargs)
        nbk.ensure_device_role(create_device_role=True)

        self.assertEqual(nbk.nb_device_role, device_role)

    def test_ensure_device_instance_not_exist(self):
        """Verify ensure_device_instance function."""
        serial_number = "123456"
        platform_slug = "cisco_ios"
        hostname = "device1"

        onboarding_kwargs = {
            "netdev_hostname": hostname,
            "netdev_nb_role_slug": PLUGIN_SETTINGS["default_device_role"],
            "netdev_nb_role_color": PLUGIN_SETTINGS["default_device_role_color"],
            "netdev_vendor": "Cisco",
            "netdev_model": "CSR1000v",
            "netdev_nb_site_slug": self.site1.slug,
            "netdev_netmiko_device_type": platform_slug,
            "netdev_serial_number": serial_number,
            "netdev_mgmt_ip_address": "192.0.2.10",
            "netdev_mgmt_ifname": "GigaEthernet0",
            "netdev_mgmt_pflen": 24,
        }

        nbk = NetboxKeeper(**onboarding_kwargs)

        nbk.ensure_device()

        self.assertIsInstance(nbk.device, Device)
        self.assertEqual(nbk.device.name, hostname)
        self.assertEqual(nbk.device.status, PLUGIN_SETTINGS["default_device_status"])
        self.assertEqual(nbk.device.platform.slug, platform_slug)
        self.assertEqual(nbk.device.serial, serial_number)

    def test_ensure_device_instance_exist(self):
        """Verify ensure_device_instance function."""
        manufacturer = Manufacturer.objects.create(name="Cisco", slug="cisco")

        device_role = DeviceRole.objects.create(name="Switch", slug="switch")

        device_type = DeviceType.objects.create(slug="c2960", model="c2960", manufacturer=manufacturer)

        device_name = "test_name"

        device = Device.objects.create(
            name=device_name,
            site=self.site1,
            device_type=device_type,
            device_role=device_role,
            status="planned",
            serial="987654",
        )

        onboarding_kwargs = {
            "netdev_hostname": device_name,
            "netdev_nb_role_slug": "switch",
            "netdev_vendor": "Cisco",
            "netdev_model": "c2960",
            "netdev_nb_site_slug": self.site1.slug,
            "netdev_netmiko_device_type": "cisco_ios",
            "netdev_serial_number": "123456",
            "netdev_mgmt_ip_address": "192.0.2.10",
            "netdev_mgmt_ifname": "GigaEthernet0",
            "netdev_mgmt_pflen": 24,
        }

        nbk = NetboxKeeper(**onboarding_kwargs)

        nbk.ensure_device()

        self.assertIsInstance(nbk.device, Device)
        self.assertEqual(nbk.device.pk, device.pk)

        self.assertEqual(nbk.device.name, device_name)
        self.assertEqual(nbk.device.platform.slug, "cisco_ios")
        self.assertEqual(nbk.device.serial, "123456")

    def test_ensure_interface_not_exist(self):
        """Verify ensure_interface function when the interface do not exist."""
        onboarding_kwargs = {
            "netdev_hostname": "device1",
            "netdev_nb_role_slug": PLUGIN_SETTINGS["default_device_role"],
            "netdev_nb_role_color": PLUGIN_SETTINGS["default_device_role_color"],
            "netdev_vendor": "Cisco",
            "netdev_model": "CSR1000v",
            "netdev_nb_site_slug": self.site1.slug,
            "netdev_netmiko_device_type": "cisco_ios",
            "netdev_serial_number": "123456",
            "netdev_mgmt_ip_address": "192.0.2.10",
            "netdev_mgmt_ifname": "ge-0/0/0",
            "netdev_mgmt_pflen": 24,
        }

        nbk = NetboxKeeper(**onboarding_kwargs)
        nbk.ensure_device()

        self.assertIsInstance(nbk.nb_mgmt_ifname, Interface)
        self.assertEqual(nbk.nb_mgmt_ifname.name, "ge-0/0/0")

    def test_ensure_interface_exist(self):
        """Verify ensure_interface function when the interface already exist."""
        manufacturer = Manufacturer.objects.create(name="Cisco", slug="cisco")

        device_role = DeviceRole.objects.create(name="Switch", slug="switch")

        device_type = DeviceType.objects.create(slug="c2960", model="c2960", manufacturer=manufacturer)

        device_name = "test_name"
        netdev_mgmt_ifname = "GigaEthernet0"

        device = Device.objects.create(
            name=device_name,
            site=self.site1,
            device_type=device_type,
            device_role=device_role,
            status="planned",
            serial="987654",
        )

        intf = Interface.objects.create(name=netdev_mgmt_ifname, device=device)

        onboarding_kwargs = {
            "netdev_hostname": device_name,
            "netdev_nb_role_slug": "switch",
            "netdev_vendor": "Cisco",
            "netdev_model": "c2960",
            "netdev_nb_site_slug": self.site1.slug,
            "netdev_netmiko_device_type": "cisco_ios",
            "netdev_serial_number": "123456",
            "netdev_mgmt_ip_address": "192.0.2.10",
            "netdev_mgmt_ifname": netdev_mgmt_ifname,
            "netdev_mgmt_pflen": 24,
        }

        nbk = NetboxKeeper(**onboarding_kwargs)

        nbk.ensure_device()

        self.assertEqual(nbk.nb_mgmt_ifname, intf)

    def test_ensure_primary_ip_not_exist(self):
        """Verify ensure_primary_ip function when the IP address do not already exist."""
        onboarding_kwargs = {
            "netdev_hostname": "device1",
            "netdev_nb_role_slug": PLUGIN_SETTINGS["default_device_role"],
            "netdev_nb_role_color": PLUGIN_SETTINGS["default_device_role_color"],
            "netdev_vendor": "Cisco",
            "netdev_model": "CSR1000v",
            "netdev_nb_site_slug": self.site1.slug,
            "netdev_netmiko_device_type": "cisco_ios",
            "netdev_serial_number": "123456",
            "netdev_mgmt_ip_address": "192.0.2.10",
            "netdev_mgmt_ifname": "ge-0/0/0",
            "netdev_mgmt_pflen": 24,
        }

        nbk = NetboxKeeper(**onboarding_kwargs)
        nbk.ensure_device()

        self.assertIsInstance(nbk.nb_primary_ip, IPAddress)
        self.assertIn(nbk.nb_primary_ip, Interface.objects.get(device=nbk.device, name="ge-0/0/0").ip_addresses.all())
        self.assertEqual(nbk.device.primary_ip, nbk.nb_primary_ip)

    def test_ensure_device_platform_missing(self):
        """Verify ensure_device_platform function when Platform object is not present."""
        platform_name = "cisco_ios"

        onboarding_kwargs = {
            "netdev_hostname": "device1",
            "netdev_nb_role_slug": PLUGIN_SETTINGS["default_device_role"],
            "netdev_vendor": "Cisco",
            "netdev_model": "CSR1000v",
            "netdev_nb_site_slug": self.site1.slug,
            "netdev_nb_platform_slug": platform_name,
            "netdev_netmiko_device_type": platform_name,
        }

        nbk = NetboxKeeper(**onboarding_kwargs)

        with self.assertRaises(OnboardException) as exc_info:
            nbk.ensure_device_platform(create_platform_if_missing=False)
            self.assertEqual(exc_info.exception.message, f"ERROR device platform not found: {platform_name}")
            self.assertEqual(exc_info.exception.reason, "fail-config")

        nbk.ensure_device_platform(create_platform_if_missing=True)
        self.assertIsInstance(nbk.nb_platform, Platform)
        self.assertEqual(nbk.nb_platform.slug, slugify(platform_name))

    def test_ensure_platform_present(self):
        """Verify ensure_device_platform function when Platform object is present."""
        platform_name = "juniper_junos"

        manufacturer = Manufacturer.objects.create(name="Juniper", slug="juniper")

        device_type = DeviceType.objects.create(slug="srx3600", model="SRX3600", manufacturer=manufacturer)

        platform = Platform.objects.create(slug=platform_name, name=platform_name,)

        onboarding_kwargs = {
            "netdev_hostname": "device2",
            "netdev_nb_role_slug": PLUGIN_SETTINGS["default_device_role"],
            "netdev_vendor": "Juniper",
            "netdev_nb_device_type_slug": device_type.slug,
            "netdev_nb_site_slug": self.site1.slug,
            "netdev_nb_platform_slug": platform_name,
        }

        nbk = NetboxKeeper(**onboarding_kwargs)

        nbk.ensure_device_platform(create_platform_if_missing=False)

        self.assertIsInstance(nbk.nb_platform, Platform)
        self.assertEqual(nbk.nb_platform, platform)
        self.assertEqual(nbk.nb_platform.slug, slugify(platform_name))

    def test_platform_map(self):
        """Verify platform mapping of netmiko to slug functionality."""
        # Create static mapping
        PLUGIN_SETTINGS["platform_map"] = {"cisco_ios": "ios", "arista_eos": "eos", "cisco_nxos": "cisco-nxos"}

        onboarding_kwargs = {
            "netdev_hostname": "device1",
            "netdev_nb_role_slug": PLUGIN_SETTINGS["default_device_role"],
            "netdev_vendor": "Cisco",
            "netdev_model": "CSR1000v",
            "netdev_nb_site_slug": self.site1.slug,
            "netdev_netmiko_device_type": "cisco_ios",
        }

        nbk = NetboxKeeper(**onboarding_kwargs)

        nbk.ensure_device_platform(create_platform_if_missing=True)
        self.assertIsInstance(nbk.nb_platform, Platform)
        self.assertEqual(nbk.nb_platform.slug, slugify(PLUGIN_SETTINGS["platform_map"]["cisco_ios"]))
        self.assertEqual(
            Platform.objects.get(name=PLUGIN_SETTINGS["platform_map"]["cisco_ios"]).name,
            slugify(PLUGIN_SETTINGS["platform_map"]["cisco_ios"]),
        )



# Generated by Django 3.0.6 on 2020-05-18 21:58

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("dcim", "0105_interface_name_collation"),
    ]

    operations = [
        migrations.CreateModel(
            name="OnboardingTask",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False)),
                ("ip_address", models.CharField(max_length=255, null=True)),
                ("device_type", models.CharField(max_length=255, null=True)),
                ("status", models.CharField(max_length=255)),
                ("failed_reason", models.CharField(max_length=255, null=True)),
                ("message", models.CharField(blank=True, max_length=511)),
                ("port", models.PositiveSmallIntegerField(default=22)),
                ("timeout", models.PositiveSmallIntegerField(default=30)),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                (
                    "created_device",
                    models.ForeignKey(
                        blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to="dcim.Device"
                    ),
                ),
                (
                    "platform",
                    models.ForeignKey(
                        blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to="dcim.Platform"
                    ),
                ),
                (
                    "role",
                    models.ForeignKey(
                        blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to="dcim.DeviceRole"
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to="dcim.Site"
                    ),
                ),
            ],
            options={"ordering": ["created_on"],},
        ),
    ]



"""Onboarding Extension for IOS.

(c) 2020 Network To Code
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
  http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from netbox_onboarding.onboarding.onboarding import StandaloneOnboarding


class OnboardingDriverExtensions:
    """Onboarding Driver's Extensions."""

    def __init__(self, napalm_device):
        """Initialize class."""
        self.napalm_device = napalm_device

    @property
    def onboarding_class(self):
        """Return onboarding class for IOS driver.

        Currently supported is Standalone Onboarding Process.

        Result of this method is used by the OnboardingManager to
        initiate the instance of the onboarding class.
        """
        return StandaloneOnboarding

    @property
    def ext_result(self):
        """This method is used to store any object as a return value.

        Result of this method is passed to the onboarding class as
        driver_addon_result argument.

        :return: Any()
        """
        return None



"""Model serializers for the netbox_onboarding REST API.

(c) 2020 Network To Code
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
  http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from rest_framework import serializers
from django_rq import get_queue

from dcim.models import Site, DeviceRole, Platform

from netbox_onboarding.models import OnboardingTask
from netbox_onboarding.utils.credentials import Credentials


class OnboardingTaskSerializer(serializers.ModelSerializer):
    """Serializer for the OnboardingTask model."""

    site = serializers.SlugRelatedField(
        many=False,
        read_only=False,
        queryset=Site.objects.all(),
        slug_field="slug",
        required=True,
        help_text="NetBox site 'slug' value",
    )

    ip_address = serializers.CharField(required=True, help_text="IP Address to reach device",)

    username = serializers.CharField(required=False, write_only=True, help_text="Device username",)

    password = serializers.CharField(required=False, write_only=True, help_text="Device password",)

    secret = serializers.CharField(required=False, write_only=True, help_text="Device secret password",)

    port = serializers.IntegerField(required=False, help_text="Device PORT to check for online")

    timeout = serializers.IntegerField(required=False, help_text="Timeout (sec) for device connect")

    role = serializers.SlugRelatedField(
        many=False,
        read_only=False,
        queryset=DeviceRole.objects.all(),
        slug_field="slug",
        required=False,
        help_text="NetBox device role 'slug' value",
    )

    device_type = serializers.CharField(required=False, help_text="NetBox device type 'slug' value",)

    platform = serializers.SlugRelatedField(
        many=False,
        read_only=False,
        queryset=Platform.objects.all(),
        slug_field="slug",
        required=False,
        help_text="NetBox Platform 'slug' value",
    )

    created_device = serializers.CharField(required=False, read_only=True, help_text="Created device name",)

    status = serializers.CharField(required=False, read_only=True, help_text="Onboarding Status")

    failed_reason = serializers.CharField(required=False, read_only=True, help_text="Failure reason")

    message = serializers.CharField(required=False, read_only=True, help_text="Status message")

    class Meta:  # noqa: D106 "Missing docstring in public nested class"
        model = OnboardingTask
        fields = [
            "id",
            "site",
            "ip_address",
            "username",
            "password",
            "secret",
            "port",
            "timeout",
            "role",
            "device_type",
            "platform",
            "created_device",
            "status",
            "failed_reason",
            "message",
        ]

    def create(self, validated_data):
        """Create an OnboardingTask and enqueue it for processing."""
        # Fields are string-type so default to empty (instead of None)
        username = validated_data.pop("username", "")
        password = validated_data.pop("password", "")
        secret = validated_data.pop("secret", "")

        credentials = Credentials(username=username, password=password, secret=secret,)

        ot = OnboardingTask.objects.create(**validated_data)

        webhook_queue = get_queue("default")

        webhook_queue.enqueue("netbox_onboarding.worker.onboard_device", ot.id, credentials)

        return ot
